import { DependencyContainer } from "tsyringe";

import { ConfigServer } from "@spt-aki/servers/ConfigServer";
import { ConfigTypes } from "@spt-aki/models/enums/ConfigTypes";
import { DatabaseServer } from "@spt-aki/servers/DatabaseServer";
import { IPostDBLoadMod } from "@spt-aki/models/external/IPostDBLoadMod";
import { ITraderConfig } from "@spt-aki/models/spt/config/ITraderConfig";
import { SecuredContainers } from "@spt-aki/models/enums/ContainerTypes";
import { IRagfairConfig  } from "@spt-aki/models/spt/config/IRagfairConfig";


class KHCM implements IPostDBLoadMod {

    postDBLoad(container: DependencyContainer): void {
        const logger = container.resolve<ILogger>("WinstonLogger");
        const databaseServer = container.resolve<DatabaseServer>("DatabaseServer");
        const configServer = container.resolve<ConfigServer>("ConfigServer");
        const traderConfig = configServer.getConfig<ITraderConfig>(ConfigTypes.TRADER);
		const ragfairConfig = configServer.getConfig<IRagfairConfig >(ConfigTypes.RAGFAIR);

		//allowed: wifi-cam, ms2000 marker, signal jammer, rangefinder, compass, multitool, roubles, dollars, euros, maps, mp5 suppressor, documents case
		const whitelist = ["5b4391a586f7745321235ab2","5991b51486f77447b112d44f","5ac78a9b86f7741cca0bbd8d","61605e13ffa6e502ac5e7eef","5f4f9eb969cdc30ff33f09db","544fb5454bdc2df8738b456a","5449016a4bdc2d6f028b456f","5696686a4bdc2da3298b456a","569668774bdc2da2298b4568","574eb85c245977648157eec3","5798a2832459774b53341029","5be4038986f774527d3fae60","5a80a29286f7742b25692012","5a8036fb86f77407252ddc02","5900b89686f7744e704a8747","5926d33d86f77410de68ebc0","590c60fc86f77412b13fddcf"];
		const whitelistParent = ["5795f317245977243854e041"]; //cases
		
        const tables = databaseServer.getTables();
        const items = tables.templates.items;
        const traders = tables.traders;
        const prices = tables.templates.prices;
        const hideoutareas = tables.hideout.areas;
		
		// find all items that have allowed parents
		for (let item in items) {
			if (whitelistParent.includes(items[item]._parent)) {
				//logger.info(item + ": " + items[item]._parent);
				whitelist.push(item);
			}
		}
		
		// disable requirements for areas that require Stash lvl 2 (workbench#3, nutrition#3)
        for (let area of hideoutareas) {
			if (area.type == 3)
				continue;
            for (let stage in area.stages) {
				for (let req in area.stages[stage].requirements) {
					let at = area.stages[stage].requirements[req].areaType;
					if (at != null) {
						if (at == 3) {
							area.stages[stage].requirements[req].requiredLevel = 1;
						}
					}
				}
			}
		}
				
		// no Scav runs (well, one every 100 hrs)
		tables.globals.config.SavagePlayCooldown = 360000;
		
		// flea market overpriced  (kept open, so wishlists can be used)
		// !!! disable selling on ragfair
        tables.globals.config.RagFair.minUserLevel = 0;
		ragfairConfig.dynamic.offerItemCount.min = 1;
		ragfairConfig.dynamic.offerItemCount.max = 2;
		for (let item in prices) {
			if (!whitelist.includes(item)) {
				let newprice = Math.min(prices[item] * 50, 90000000);
				//logger.info(item + ": " + prices[item] + ", " + newprice);
				prices[item] = newprice;
			}
		}
		ragfairConfig.sell.chance.base = 0;

		// no insurance, no sales
		for (let tid in traders) {
			let trader = traders[tid];

			trader.base.insurance.availability = false;

			if (trader.assort != null) {
				const assortItems = trader.assort.items;
				if (assortItems.length != null) {
					for (let item of assortItems) {
						if !(whitelist.includes(item._tpl) {
							item.upd = {"UnlimitedCount":false,"StackObjectsCount":0};
						}
					}
				}
			}

		}
		// fence has nothing
		traderConfig.fence.assortSize = 0;		
		
		// no stash upgrades
		const hideout = tables.hideout.areas;
        for (let area of hideout) {
			if (area.type==3) {
				area.enabled = false;
			}
        }
		
		// single cell secure container
        for (let [key, value] of Object.entries(SecuredContainers)) {
            items[value]._props.Grids[0]._props.cellsH = 1;
            items[value]._props.Grids[0]._props.cellsV = 1;
        }
		
    }

}
    
module.exports = { mod: new KHCM() }